;#include "ti86asm.inc "
; External data loader created by Christophe Tremblay 						ti_chris@yahoo.com
; Compatible with the TI-85, TI-86

; - Please define the calculator you wish to use.  By default, it uses the TI-86 structure
;   Possible values are : TI-86, TI-85, TI-83+, TI-83, TI-82
; - Make sure the type you're looking for matches the current setting
; - Also make sure you defined your IDWord which will identify which strings should be taken
; - It return AHL = Level Address
; - Definen it callable to make the routine callable or else use the #include command where you 
;   need to code to be appended to

; #define Callable
; #define TI-85
; #define TI-83
; #define TI-82 

VReal	 equ 00h
VComplex equ 01h
VVectorR equ 02h
VVectorC equ 03h
VListR	 equ 04h
VListC	 equ 05h
VMatrixR equ 06h
VMatrixC equ 07h
VConstR	 equ 08h
VConstC	 equ 09h
VEquat	 equ 0Ah
VString  equ 0Ch
VGBDFun	 equ 0Dh
VGBDPol	 equ 0Eh
VGBDPar	 equ 0Fh
VGBDDif	 equ 10h
VPic	 equ 11h
VProgram equ 12h

LEEOV	equ 0D29Bh									; the End of VAT routine

;.org _asm_exec_ram
LELoadData:
	ld   c, 0
	exx												; Set c = 0 in the shadow registers
#ifdef TI85
	ld   a, %01000011								; Load page 3
#else
	ld   a, %01000111								; Load page 7
#endif
	out  (6), a										; Retrieve the VAT into 08000h

; The first step is to get all the names of the variables we'll be browsing...
	ld   hl, 0BFFFh									; point to the end of the VAT
	ld   bc, 0
	ld   de, LevName								; de = Where we'll store the names

LeSeek:
	add  hl, bc
	ld   bc, 5

	ld   a, (hl)									; Get the variable type
	sbc  hl, bc

	ld   c, (hl)
	inc  c
	cp   LESeek										; Is the filetype what we seek?
	jr   nz, LESkipVar

	push hl
	exx												; Preserve
	pop  hl
	inc  hl											; Skip the name length
	inc  hl
	ld   b, (hl)
	inc  hl
	ld   d, (hl)
	inc  hl
	ld   e, (hl)									; BDE = Level Address

	call _ex_ahl_bde								; Exchanges ahl and bde
	call _ahl_plus_2_pg3							; AHL = AHL + 2 (Skip the var size)
	call _get_word_ahl								; de = (AHL)

#ifdef TI85
	ld   a, %01000011								; Load page 3
#else
	ld   a, %01000111								; Load page 7
#endif
	out  (6), a										; Retrieve the VAT into 08000h

	ld   hl, IDWord
	and  a
	sbc  hl, de
	jr   nz, LENotOfType							; If they're the same then it's all good =)

	inc  c											; c = Total number of Files found so far!
	exx

LECopyName:
	ld   a, (hl)
	ld   (de), a
	dec  hl											; Skip the size
	inc  de
	dec  c
	jr   nz, LECopyName								; The name is saved using a reversed techni.
	jr   LECheckDone

LENotOfType:
	exx												; Restore all the registers

LESkipVar:
	and  a	
	sbc  hl, bc										; Skip the name itself

LECheckDone:
	ld   bc, (LEEOV)
	sbc  hl, bc
	jr   nz, LeSeek

	exx												; c = Max


; At this point all the levels we could load should have their name in the data buffer (hl)
	xor  a
	cp   c
	jr   nz, LEDoMenu

	scf
#ifdef Callable
	ret
#else
	jp   LEReturn
#endif

LEDoMenu:
	ld   (LEWindow), a
	ld   (LEAt), a

LEDrawChoice:
	ld   a, c										; Preserve c
	call _clrLCD
	ld  c, a

	ld   hl, LevName
LEWindow	equ $ + 1								; The number of frames to go down
	ld   a, 0
	or   a
	jr   z, LENoSkip

	ld   b, a
	ld   d, 0
LESkip:
	ld   e, (hl)
	inc  e											; de = Name Length
	add  hl, de
	djnz LESkip

LENoSkip:
	ld   b, 64 / 6									; b = Repeat
	ld   a, c										; c = Max
	cp   b
	jp   p, LELots					; FIXME : Conver to nc

	ld   e, c										; There's not enough files to fill a screen

LELots:
	xor  a
	ld   (_penRow), a								; Initialise the drawing pointer at 0

LERow:
	xor  a
	ld   (_penCol), a

	ld   b, (hl)
	inc  hl
	call _vputsn									; Write the filename

	ld   a, (_penRow)
	add  a, 6
	ld   (_penRow), a								; Skip to the next variable
	dec  e
	jr   nz, LERow

	ld   a, (LEWindow)
	ld   b, a
LEAt	 equ $ + 1
	ld   a, 0
	sub  b											; a = LEAt - LEWindow


; Here we're starting the shading procedure
	ld   h, 0
	add  a, a
	add  a, a
	add  a, a
	add  a, a
	rla
	rl   h
	ld   l, a
	ld   d, h
	ld   e, l
	add  hl, hl
	add  hl, de										; a * 16 * 6
	ld   de, 0FC00h
	add  hl, de

	ld   e, 16 * 7

LEShade:
	ld   a, (hl)
	cpl
	ld   (hl), a
	inc  hl
	dec  e
	jr   nz, LEShade

LELeechKey:
	call _get_key

	cp   K_UP
	jr   nz, LENotUp								; Is the up key pressed?

	ld   a, (LEAt)
	or   a
	jr   z, LEDrawChoice

	dec  a
	ld   (LEAt), a									; Decrement LEAt if we can
	ld   a, (LEWindow)
	or   a
	jr   z, LEDrawChoice

	dec  a
	ld   (LEWindow), a								; Decrement the window if necesary
	jr   LEDrawChoice

LENotUp:
	cp   K_DOWN
	jr   nz, LENotDown								; Is the down key pressed?

	ld   a, (LEAt)
	inc  a
	cp   c											; Are we at the end?
	jr   z, LEDrawChoice

	ld   (LEAt), a
	sub  64 / 6
	ld   b, a
	ld   a, (LEWindow)
	cp   b
	jp   nz, LEDrawChoice

	inc  a
	ld   (LEWindow), a
	jp   LEDrawChoice

LENotDown:
	cp   K_ENTER
	jr   nz, LELeechKey								; Is the enter key pressed?

	ld   hl, LevName
	ld   a, (LEAt)
	ld   b, a
	or   a
	jr   z, LENoSkip2

	ld   d, 0
LESkip2:
	ld   e, (hl)
	inc  e
	add  hl, de
	djnz LESkip2

LENoSkip2:
	dec  hl
	ld   (hl), VString
	rst  20h										; Copy 10bytes from hl to OP1
	rst  10h										; Find name and place it's address in AHL
	call _ex_ahl_bde								; AHL = BDE

	and  a											; Clear the carry flag

LEReturn:
