
; Created by Christopher Tremblay   										"ti_chris@yahoo.com"
; Bidirectional port transfer

; Public functions call :
;
; Byte CheckID();
; Byte Transfer(Byte Data, Byte Length);

Slave0	equ %11111100										; 0 = 00b
Master0	equ %11101000										; 1 = 01b
Slave1	equ %11010100										; 2 = 10b
Master1	equ %11000000										; 3 = 11b

Normal	equ %11000000										; Typical Wires state (On, On) = 11b
Free	equ %11111100										; Reversed state (Off, Off)	   = 00b



; Small patched procedure ran by all 3 routines to check if the Exit key has been pressed
; if so, the procedure quits and the program should "wreckognise"=)that the key has been pressed
; and workout the proper course
ChkKey:
	dec  hl
	ld   a, h
	or   l

; I think this used to generate problems since the port reads were too quick
;	ld   a, ExitRow
;	out  (1), a
;	in   a, (1)
;	in   a, (1)
;	bit  ExitKey, a
KeyPatch equ $ + 1
	jr   nz, MWaitState
	ret  													; If Exit is pressed, return


; Procedure to check which calc is the master.  The fastest one will be designed as the Master
; This is the handshake which shouldn't be called more then once per game
; [In]   : NULL
; [Out]  : d = 0 (Master), d = 1 (Slave)
; [Uses] : af
CheckID:
	ld   hl, 30000
	in   a, (7)
	and  3
	jr   nz, UsMaster										; Other calc is already on, = Slave

	ld   a, Normal											; Shutoff all the wires
	ld   bc, (SWaitState - KeyPatch - 1) * 100h + BiDiSlave - BiDiMaster
	ld   d, 1												; Identifies as Slave
	jr   SetVal

UsMaster:
	ld   a, Free											; Shut both off to show we're the 1
	ld   bc, (CheckSlave - KeyPatch - 1) * 100h
	call SetVal

CheckSlave:
	in   a, (7)
	and  %00000011
	jr   nz, ChkKey

	ld   b, MWaitState - KeyPatch - 1
	ld   a, Free
	ld   d, 0												; Identifies as Master

SetVal:
	out  (7), a
	ld   a, b
	ld   (KeyPatch), a										; Set as Master
	ld   a, c
	ld   (Transfer + 1), a
	ret														; We're the slave, return 0


; Procedure to send & receive from anyone's point of view
; [In]   : c = Data, b = Length
; [Out]  : d = Data, bc = 0
; [Uses] : af
Transfer:
	ld   hl, 30000											; Setup our timer!
	jr   BiDiMaster


; Procedure to send and receive a string of bits with the port from a Master's point of view!
; [In]   : c = Data, b = Length
; [Out]  : d = Data, bc = 0, a = 0C0h
; [Uses] : f
BiDiMaster:
	ld   d, 0
SendMoreMaster:
	srl  c
	jr   c, GiveOne

	ld   a, Master0
	out  (7), a
	jr   MWaitState											; Send "0"

GiveOne:
	ld   a, Master1
	out  (7), a												; Send "1"

MWaitState:
	in   a, (7)												; Did we get a reply?
	rra
	jr   c, ChkKey											; Nothing, check for Exit

	rra														; Put the info bit in the carry
	rl   d
	djnz SendMoreMaster

	ld   a, Normal
	out  (7), a												; Clear out the sigal
	ret


; Procedure to send and receive a string of bits with the port from a Slave's point of view!
; [In]   : c = Data, b = Length
; [Out]  : d = Data, bc = 0
; [Uses] : af
BiDiSlave:
	ld   d, 0
SWaitState:
	in   a, (7)												; Is the master trying to send stuff
	rra
	jr   nc, ChkKey											; Nothing, check for Exit

	rra
	rl   d

	srl  c
	jr   nc, GiveZero

	ld   a, Slave1
	out  (7), a
	djnz SWaitState											; Send "1"
	ret

GiveZero:
	ld   a, Slave0
	out  (7), a												; Send "0"
	djnz SWaitState
	ret


; Explanation of the bidirectional technology : 
;
; The advantage of this technique is that you can send and receive data in one single transfer
; compared to the other techniques which allow only Sending or Receiving.
; I've also compared with the unidirectional routines and this one here seems to be more 
; optimized.
;
; Master/Slave identification :
; The Master is the first calculator to run the program, it will identify itself as the master
; by reversing the initial wire state (Both will be off).  The slave will identify itself by 
; checking wether or not the wires are not in Normal mode (Both On).  Once identified, it will
; set the wires back to the normal mode (Both On)
;
; The protocol itself :
; The Master will be the first calc to send data and therefore also the last calc to receive
; data.  It sends it's data by puting .  For speed reasons, the master's (0, 1) state equals
; (Both Wires Off, Red On) which results in the other calc receiving (01b, 11b).  We'll be
; using the later expression  to simplify the explanation.  The other calc first looks to see
; if the least significant bit of what it receives is on.  If it is, then it knows it just 
; received data from the master and takes following bit as the data.  As for the Slave, it 
; aswell has (0, 1) state both in it's case, it's (White On, Both Wires On) and we'll be using
; (00b, 10b), the received string to explain.  The Master, after sending it's data has set the
; least significant bit on, now it'll check if it's back to 0 which means the slave replied.  If
; it's is off, it takes the following bit as the data.  This process keeps looping until it
; reaches the amounth of bit which was supposed to be sent.  At the end of the trasfer, the 
; Master will set the wires back to it's "normal" mode which is both wires On to make sure that
; if the Slave calc runs it's receiving code before the master outputs anything, it'll have the
; first bit cleared so it won't check the data.

